
/* Chrysalide - Outil d'analyse de fichiers binaires
 * processor.c - manipulation du processeur de la VM Dalvik
 *
 * Copyright (C) 2010-2017 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Chrysalide.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "processor.h"


#include <assert.h>


#include <plugins/dex/format.h>


#include "instruction.h"
#include "processor-int.h"
#include "pseudo/fill.h"
#include "pseudo/identifiers.h"
#include "pseudo/switch.h"



/* Initialise la classe des processeurs de VM Dalvik. */
static void g_dalvik_processor_class_init(GDalvikProcessorClass *);

/* Initialise une instance de processeur de VM Dalvik. */
static void g_dalvik_processor_init(GDalvikProcessor *);

/* Supprime toutes les références externes. */
static void g_dalvik_processor_dispose(GDalvikProcessor *);

/* Procède à la libération totale de la mémoire. */
static void g_dalvik_processor_finalize(GDalvikProcessor *);

/* Fournit un contexte pour l'exécution du processeur Dalvik. */
static GDalvikContext *g_dalvik_processor_get_context(const GDalvikProcessor *);

/* Fournit un contexte pour la décompilation Dalvik. */
//static GDalvikDContext *g_dalvik_processor_get_decomp_context(const GDalvikProcessor *);



/* Indique le type défini par la GLib pour le processeur DALVIK. */
G_DEFINE_TYPE(GDalvikProcessor, g_dalvik_processor, G_TYPE_ARCH_PROCESSOR);


/******************************************************************************
*                                                                             *
*  Paramètres  : klass = classe à initialiser.                                *
*                                                                             *
*  Description : Initialise la classe des processeurs de VM Dalvik.           *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_dalvik_processor_class_init(GDalvikProcessorClass *klass)
{
    GObjectClass *object;                   /* Autre version de la classe  */
    GArchProcessorClass *proc;              /* Encore une autre vision...  */

    object = G_OBJECT_CLASS(klass);

    object->dispose = (GObjectFinalizeFunc/* ! */)g_dalvik_processor_dispose;
    object->finalize = (GObjectFinalizeFunc)g_dalvik_processor_finalize;

    proc = G_ARCH_PROCESSOR_CLASS(klass);

    proc->get_ctx = (get_processor_context_fc)g_dalvik_processor_get_context;
    //proc->get_dec_ctx = (get_decomp_context_fc)g_dalvik_processor_get_decomp_context;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : proc = instance à initialiser.                               *
*                                                                             *
*  Description : Initialise une instance de processeur de VM Dalvik.          *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_dalvik_processor_init(GDalvikProcessor *proc)
{
    GArchProcessor *parent;                 /* Instance parente            */

    parent = G_ARCH_PROCESSOR(proc);

    parent->endianness = SRE_LITTLE;
    parent->memsize = MDS_32_BITS;
    parent->inssize = MDS_16_BITS;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : proc = instance d'objet GLib à traiter.                      *
*                                                                             *
*  Description : Supprime toutes les références externes.                     *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_dalvik_processor_dispose(GDalvikProcessor *proc)
{
    G_OBJECT_CLASS(g_dalvik_processor_parent_class)->dispose(G_OBJECT(proc));

}


/******************************************************************************
*                                                                             *
*  Paramètres  : bookmark = instance d'objet GLib à traiter.                  *
*                                                                             *
*  Description : Procède à la libération totale de la mémoire.                *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_dalvik_processor_finalize(GDalvikProcessor *proc)
{
    G_OBJECT_CLASS(g_dalvik_processor_parent_class)->finalize(G_OBJECT(proc));

}


/******************************************************************************
*                                                                             *
*  Paramètres  : proc = architecture, spectatrice ici.                        *
*                                                                             *
*  Description : Fournit un contexte pour l'exécution du processeur Dalvik.   *
*                                                                             *
*  Retour      : Contexte mis en place.                                       *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static GDalvikContext *g_dalvik_processor_get_context(const GDalvikProcessor *proc)
{
    return g_dalvik_context_new();

}


/******************************************************************************
*                                                                             *
*  Paramètres  : proc = architecture, spectatrice ici.                        *
*                                                                             *
*  Description : Fournit un contexte pour la décompilation Dalvik.            *
*                                                                             *
*  Retour      : Contexte mis en place.                                       *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/
#if 0
static GDalvikDContext *g_dalvik_processor_get_decomp_context(const GDalvikProcessor *proc)
{
    return g_dalvik_dcontext_new();

}
#endif


/******************************************************************************
*                                                                             *
*  Paramètres  : proc = architecture visée par la procédure.                  *
*                data = flux de données à analyser.                           *
*                pos  = position courante dans ce flux. [OUT]                 *
*                end  = limite des données à analyser.                        *
*                addr = adresse virtuelle de l'instruction.                   *
*                low8 = 8 bits de poids faible déjà lus.                      *
*                                                                             *
*  Description : Décode une pseudo-instruction dans un flux de données.       *
*                                                                             *
*  Retour      : Instruction mise en place ou NULL si aucune trouvée.         *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

GArchInstruction *g_dalvik_processor_disassemble_pseudo(const GArchProcessor *proc, GDalvikContext *ctx, const GBinContent *content, vmpa2t *pos, uint8_t low8)
{
    GArchInstruction *result;               /* Instruction à renvoyer      */
    vmpa2t tmp;                             /* Position modifiable         */
    uint8_t high8;                          /* Nouvelle octet à venir lire */
    uint16_t ident;                         /* Valeur lue dans le code     */

    /* Vérification astucieuse et rapide...*/
    if (low8 != 0x00 /* DOP_NOP */)
        return NULL;

    result = NULL;

    copy_vmpa(&tmp, pos);

    if (!g_binary_content_read_u8(content, pos, &high8))
        goto gdpdp_exit;

    ident = high8 << 8 | low8;

    switch (ident)
    {
        case DPO_PACKED_SWITCH:
        case DPO_SPARSE_SWITCH:
            result = g_dalvik_switch_instr_new(ident, ctx, content, pos);
            break;

        case DPO_FILL_ARRAY_DATA:
            result = g_dalvik_fill_instr_new(ident, ctx, content, pos);
            break;

        default:
            result = NULL;
            break;

    }

 gdpdp_exit:

    if (result == NULL)
        copy_vmpa(pos, &tmp);

    return result;

}
